@@ -14,7 +14,7 @@
 # General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, see <http://www.gnu.org/licenses/>.
+# along with this program; if not, see <https://www.gnu.org/licenses/>.
 
 message () {
   echo "USAGE: $(basename "$0") [-d datadir] [-f format] [-u] <direction> [in [out]]"
@@ -28,6 +28,7 @@ message () {
   echo " -o direction     translation direction using the translation memory,"
   echo "                  by default 'direction' is used instead"
   echo " -l               lists the available translation directions and exits"
+  echo " -V               print Apertium version"
   # This is wrong and should be deprecated, but until all tools consider \0 flush without -z then this helps to have
   echo " -z               force null-flush mode on all parts of the pipe"
   echo " direction        typically, LANG1-LANG2, but see modes.xml in language data"
@@ -78,6 +79,36 @@ check_transfuse () {
   fi
 }
 
+setvar_wrap () {
+    gawk -v var="${AP_SETVAR}" '
+BEGIN           { RS="\0|\n"; }
+NR==1    && var { printf "[<STREAMCMD:SETVAR:%s>]", var }
+                { printf "%s%s", $0, RT }
+RT=="\0" && var { printf "[<STREAMCMD:SETVAR:%s>]", var }
+RT=="\0"        { fflush() }'
+}
+setvar_unwrap () {
+    # Only strip SETVAR if *we* inserted it, allow keeping it if it was manually inserted (for debugs)
+    gawk -v var="${AP_SETVAR}" '
+BEGIN           { RS="\0|\n"; }
+            var { gsub(/\[<STREAMCMD:SETVAR:[^>]*>]/, "") }
+                { printf "%s%s", $0, RT }
+RT=="\0"        { fflush() }'
+}
+
+run_mode_wblank () {
+    setvar_wrap \
+        | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") \
+               "$OPTION" \
+               "$OPTION_TAGGER" \
+        | setvar_unwrap
+}
+
+run_mode_wblank_force_flush () {
+    NULL_FLUSH=(-z)
+    run_mode_wblank
+}
+
 test_zip () {
   if ! command -v zip &>/dev/null; then
     echo "Error: Install 'zip' command in your system";
@@ -115,7 +146,7 @@ translate_latex() {
     then cat;
     else lt-tmxproc "$TMCOMPFILE";
     fi | \
-      bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+      run_mode_wblank | \
       apertium-relatex| \
       awk '{gsub("</CONTENTS-noeos>", "</CONTENTS>"); print;}' | \
       if [[ -z "$REDIR" ]]; then apertium-postlatex-raw; else apertium-postlatex-raw > "$OUT_FILE"; fi
@@ -136,7 +167,7 @@ translate_latex_raw() {
 translate_odt () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f odt "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract -f odt "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -159,7 +190,7 @@ translate_odt () {
   then cat;
   else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    run_mode_wblank | \
     apertium-reodt|\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
@@ -180,7 +211,7 @@ translate_odt () {
 translate_docx () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f docx "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract -f docx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -217,7 +248,7 @@ translate_docx () {
   then cat;
   else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    run_mode_wblank | \
     apertium-rewxml|\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
@@ -237,7 +268,7 @@ translate_docx () {
 translate_pptx () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f pptx "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract -f pptx "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -273,7 +304,7 @@ translate_pptx () {
   then cat;
   else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    run_mode_wblank | \
     apertium-repptx |\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
@@ -311,7 +342,7 @@ translate_xlsx () {
   then cat;
   else lt-tmxproc "$TMCOMPFILE";
   fi | \
-    bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+    run_mode_wblank | \
     apertium-rexlsx |\
   awk '{punto = index($0, "/>") + 3; cabeza = substr($0, 1, punto-1); cola = substr($0, punto); n1 = substr(cabeza, index(cabeza, "\"")+1); name = substr(n1, 1, index(n1, "\"")-1); gsub("[?]> ", "?>\n", cola); print cola > name;}'
   VUELVE=$(pwd)
@@ -331,7 +362,7 @@ translate_xlsx () {
 translate_html () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f html "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -339,13 +370,13 @@ translate_html () {
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
     else lt-tmxproc "$TMCOMPFILE";
-    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml; else apertium-rehtml > "$OUT_FILE"; fi
+    fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml; else apertium-rehtml > "$OUT_FILE"; fi
 }
 
 translate_htmlnoent () {
   check_transfuse
   if [[ $USE_TRANSFUSE = "yes" ]]; then
-    tf-extract -f html "$INFILE" | bash <(apertium-wblank-mode -z "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
+    tf-extract -f html "$INFILE" | run_mode_wblank_force_flush | if [[ -z "$REDIR" ]]; then tf-inject; else tf-inject > "$OUT_FILE"; fi
     return
   fi
 
@@ -353,7 +384,7 @@ translate_htmlnoent () {
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
     else lt-tmxproc "$TMCOMPFILE";
-    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
+    fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
 }
 
 translate_htmlalt () {
@@ -361,7 +392,7 @@ translate_htmlalt () {
     if [[ -z "$TRANSLATION_MEMORY_FILE" ]]; then
       cat
     else lt-tmxproc "$TMCOMPFILE";
-    fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
+    fi | run_mode_wblank | if [[ -z "$REDIR" ]]; then apertium-rehtml-noent; else apertium-rehtml-noent > "$OUT_FILE"; fi
 }
 
 translate_line () {
@@ -374,7 +405,7 @@ translate_line () {
       else
         lt-tmxproc "$TMCOMPFILE";
       fi | \
-          bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | \
+          run_mode_wblank | \
               if [[ -z "$REDIR" ]]; then
                   apertium-retxt
               else
@@ -417,7 +448,7 @@ while [[ $OPTIND -le $# ]]; do
 done
 
 
-while getopts ":uahlzf:d:m:o:n" opt; do
+while getopts ":uahlVzf:d:m:o:nH" opt; do
   case "$opt" in
     f) FORMAT=$OPTARG ;;
     d) DATADIR=$OPTARG ;;
@@ -425,12 +456,14 @@ while getopts ":uahlzf:d:m:o:n" opt; do
     o) TRANSLATION_MEMORY_DIRECTION=$OPTARG ;;
     u) UWORDS="no" ;;
     n) FORMAT_OPTIONS+=(-n) ;;
+    H) FORMAT_OPTIONS+=(-o) ;;
     a) OPTION_TAGGER="-m" ;;
     l) LIST_MODES_AND_EXIT=true ;;
+	V) echo "${apertium_version}" && exit 0 ;;
     z) NULL_FLUSH+=(-z) ;;
     h) message ;;
     \?) echo "ERROR: Unknown option $OPTARG" >&2; message >&2 ;;
-    :) echo "ERROR: $OPTARG requires an argument" >&2; message >&2 ;;
+    *) echo "ERROR: $OPTARG requires an argument" >&2; message >&2 ;;
   esac
 done
 shift $(( OPTIND-1 ))
@@ -481,14 +514,16 @@ if [[ ! -d "$DATADIR/modes" ]]; then
 fi
 
 if [[ ! -e "$DATADIR/modes/$PAIR.mode" ]]; then
-  echo -n "Error: Mode $PAIR does not exist"
-  c=$(find "$DATADIR/modes" -name '*.mode' | wc -l)
-  if [[ "$c" -le 1 ]]; then
-    echo "."
-  else
-    echo ". Try one of:"
-    list_directions
-  fi
+  {
+    echo -n "Error: Mode $PAIR does not exist"
+    c=$(find "$DATADIR/modes" -name '*.mode' | wc -l)
+    if [[ "$c" -le 1 ]]; then
+      echo "."
+    else
+      echo ". Try one of:"
+      list_directions
+    fi
+  } >&2
   exit 1
 fi
 
@@ -660,7 +695,7 @@ fi | if [[ -z "$TRANSLATION_MEMORY_FILE" ]];
     cat
   else
     lt-tmxproc "$TMCOMPFILE"
-  fi | bash <(apertium-wblank-mode "${NULL_FLUSH[@]}" "$DATADIR/modes/$PAIR.mode") "$OPTION" "$OPTION_TAGGER" | if [[ "$FORMAT" = "none" ]]; then
+  fi | run_mode_wblank | if [[ "$FORMAT" = "none" ]]; then
     if [[ -z "$REDIR" ]]; then
       cat
     else
