@@ -12,7 +12,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 #include <apertium/interchunk.h>
 #include <lttoolbox/lt_locale.h>
@@ -23,7 +23,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <apertium/string_utils.h>
+#include <lttoolbox/string_utils.h>
 #include "getopt_long.h"
 
 #ifdef _MSC_VER
@@ -31,19 +31,18 @@
 #include <fcntl.h>
 #endif
 
-using namespace Apertium;
 using namespace std;
 
 void message(char *progname)
 {
-  wcerr << "USAGE: " << basename(progname) << " [-tz] t2x preproc [input [output]]" << endl;
-  wcerr << "  t2x        t2x rules file" << endl;
-  wcerr << "  preproc    result of preprocess trules file" << endl;
-  wcerr << "  input      input file, standard input by default" << endl;
-  wcerr << "  output     output file, standard output by default" << endl;
-  wcerr << "OPTIONS" <<endl;
-  wcerr << "  -t         trace mode" << endl;
-  wcerr << "  -z         flush buffer on '\0'" << endl;
+  cerr << "USAGE: " << basename(progname) << " [-tz] t2x preproc [input [output]]" << endl;
+  cerr << "  t2x        t2x rules file" << endl;
+  cerr << "  preproc    result of preprocess trules file" << endl;
+  cerr << "  input      input file, standard input by default" << endl;
+  cerr << "  output     output file, standard output by default" << endl;
+  cerr << "OPTIONS" <<endl;
+  cerr << "  -t         trace mode" << endl;
+  cerr << "  -z         flush buffer on '\0'" << endl;
 
   exit(EXIT_FAILURE);
 }
@@ -53,32 +52,28 @@ void testfile(string const &filename)
   struct stat mybuf;
   if(stat(filename.c_str(), &mybuf) == -1)
   {
-    wcerr << "Error: can't stat file '";
-    wcerr << filename << "'." << endl;
+    cerr << "Error: can't stat file '";
+    cerr << filename << "'." << endl;
     exit(EXIT_FAILURE);
   }
 }
 
-FILE * open_input(string const &filename)
+void open_input(InputFile& input, const char* filename)
 {
-  FILE *input = fopen(filename.c_str(), "r");
-  if(!input)
-  {
-    wcerr << "Error: can't open input file '";
-    wcerr << filename.c_str() << "'." << endl;
+  if (!input.open(filename)) {
+    cerr << "Error: can't open input file '";
+    cerr << filename << "'." << endl;
     exit(EXIT_FAILURE);
   }
-
-  return input;
 }
 
-FILE * open_output(string const &filename)
+UFILE * open_output(const char* filename)
 {
-  FILE *output = fopen(filename.c_str(), "w");
+  UFILE* output = u_fopen(filename, "w", NULL, NULL);
   if(!output)
   {
-    wcerr << "Error: can't open output file '";
-    wcerr << filename.c_str() << "'." << endl;
+    cerr << "Error: can't open output file '";
+    cerr << filename << "'." << endl;
     exit(EXIT_FAILURE);
   }
   return output;
@@ -122,13 +117,15 @@ int main(int argc, char *argv[])
     }
   }
 
-  FILE *input = stdin, *output = stdout;
-  string f1, f2;
+  InputFile input;
+  UFILE* output = u_finit(stdout, NULL, NULL);
+  const char* f1;
+  const char* f2;
   switch(argc - optind + 1)
   {
     case 5:
       output = open_output(argv[argc-1]);
-      input = open_input(argv[argc-2]);
+      open_input(input, argv[argc-2]);
       testfile(argv[argc-3]);
       testfile(argv[argc-4]);
       f1 = argv[argc-4];
@@ -136,7 +133,7 @@ int main(int argc, char *argv[])
       break;
 
     case 4:
-      input = open_input(argv[argc-1]);
+      open_input(input, argv[argc-1]);
       testfile(argv[argc-2]);
       testfile(argv[argc-3]);
       f1 = argv[argc-3];
@@ -155,11 +152,6 @@ int main(int argc, char *argv[])
       break;
   }
 
-#ifdef _MSC_VER
-  _setmode(_fileno(input), _O_U8TEXT);
-  _setmode(_fileno(output), _O_U8TEXT);
-#endif
-
   i.read(f1, f2);
   i.interchunk(input, output);
   return EXIT_SUCCESS;
