@@ -13,11 +13,10 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "getopt_long.h"
-#include <apertium/utf_converter.h>
 #include <apertium/file_morpho_stream.h>
 #include <apertium/tsx_reader.h>
 #include <apertium/tagger_data_hmm.h>
@@ -25,7 +24,7 @@
 #include <iostream>
 
 #include <cstdlib>
-#include <apertium/string_utils.h>
+#include <lttoolbox/string_utils.h>
 
 
 using namespace std;
@@ -36,13 +35,13 @@ bool check_ambclasses;
 
 void check_file(FILE *f, const string& path) {
   if (!f) {
-    wcerr<<"Error: cannot open file '"<<path<<"'\n";
+    cerr<<"Error: cannot open file '"<<path<<"'\n";
     exit(EXIT_FAILURE);
   }
 }
 
-void readwords (FILE *is, int corpus_length) {
-  FileMorphoStream lexmorfo(is, true, &tagger_data_hmm);
+void readwords (int corpus_length) {
+  FileMorphoStream lexmorfo(NULL, true, &tagger_data_hmm);
   TaggerWord *word=NULL;
   int nwords=0;
 
@@ -50,15 +49,15 @@ void readwords (FILE *is, int corpus_length) {
   while(word) {
     nwords++;
 
-    cout<<UtfConverter::toUtf8(word->get_superficial_form())<<" "<<UtfConverter::toUtf8(word->get_string_tags())<<"\n";
+    cout << word->get_superficial_form() << " " << word->get_string_tags() << "\n";
 
     if (check_ambclasses) {
       int k=tagger_data_hmm.getOutput()[word->get_tags()];
 
       if ((k>=tagger_data_hmm.getM())||(k<0)) {
-	wcerr<<"Error: Ambiguity class number out of range: "<<k<<"\n";
-	wcerr<<"Word: "<<UtfConverter::toUtf8(word->get_superficial_form())<<"\n";
-	wcerr<<"Ambiguity class: "<<UtfConverter::toUtf8(word->get_string_tags())<<"\n";
+	cerr<<"Error: Ambiguity class number out of range: "<<k<<"\n";
+	cerr<<"Word: "<< word->get_superficial_form() << "\n";
+	cerr<<"Ambiguity class: "<< word->get_string_tags() << "\n";
       }
     }
 
@@ -69,15 +68,15 @@ void readwords (FILE *is, int corpus_length) {
 
     word=lexmorfo.get_next_word();
   }
-  wcerr<<nwords<<" were readed\n";
+  cerr<<nwords<<" were readed\n";
 }
 
 
 void help(char *name) {
-  wcerr<<"USAGE:\n";
-  wcerr<<name<<" {--tsxfile file.tsx | --probfile file.prob} [--clength <corpus_length>] < file.crp \n\n";
+  cerr<<"USAGE:\n";
+  cerr<<name<<" {--tsxfile file.tsx | --probfile file.prob} [--clength <corpus_length>] < file.crp \n\n";
 
-  wcerr<<"ARGUMENTS: \n"
+  cerr<<"ARGUMENTS: \n"
       <<"   --tsxfile|-x: Specify a tagger specification file\n"
       <<"   --probfile|-p: Specify a tagger parameter file\n"
       <<"   --clength|-l: Specify the length of the corpus to process\n";
@@ -85,6 +84,7 @@ void help(char *name) {
 
 
 int main(int argc, char* argv[]) {
+  LtLocale::tryToSetLocale();
   string tsxfile="";
   string probfile="";
   int corpus_length=-1;
@@ -92,12 +92,12 @@ int main(int argc, char* argv[]) {
   int c;
   int option_index=0;
 
-  wcerr<<"LOCALE: "<<setlocale(LC_ALL,"")<<"\n";
+  cerr<<"LOCALE: "<<setlocale(LC_ALL,"")<<"\n";
 
-  wcerr<<"Command line: ";
+  cerr<<"Command line: ";
   for(int i=0; i<argc; i++)
-    wcerr<<argv[i]<<" ";
-  wcerr<<"\n";
+    cerr<<argv[i]<<" ";
+  cerr<<"\n";
 
   while (true) {
     static struct option long_options[] =
@@ -118,7 +118,7 @@ int main(int argc, char* argv[]) {
     case 'l':
       corpus_length=atoi(optarg);
       if(corpus_length<=0) {
-	wcerr<<"Error: corpus length provided with --clength must be a positive integer\n";
+	cerr<<"Error: corpus length provided with --clength must be a positive integer\n";
 	help(argv[0]);
 	exit(EXIT_FAILURE);
       }
@@ -134,8 +134,8 @@ int main(int argc, char* argv[]) {
       exit(EXIT_SUCCESS);
       break;
     case 'v':
-      wcerr<<"apertium-tagger-readwords\n";
-      wcerr<<"LICENSE:\n\n"
+      cerr<<"apertium-tagger-readwords\n";
+      cerr<<"LICENSE:\n\n"
 	  <<"   Copyright (C) 2006 Felipe Sánchez Martínez\n\n"
 	  <<"   This program is free software; you can redistribute it and/or\n"
 	  <<"   modify it under the terms of the GNU General Public License as\n"
@@ -147,7 +147,7 @@ int main(int argc, char* argv[]) {
 	  <<"   General Public License for more details.\n"
 	  <<"\n"
 	  <<"   You should have received a copy of the GNU General Public License\n"
-	  <<"   along with this program; if not, see <http://www.gnu.org/licenses/>.\n";
+	  <<"   along with this program; if not, see <https://www.gnu.org/licenses/>.\n";
       exit(EXIT_SUCCESS);
       break;
     default:
@@ -158,38 +158,38 @@ int main(int argc, char* argv[]) {
   }
 
   if((tsxfile=="") && (probfile=="")) {
-    wcerr<<"Error: You have provided neither a tagger specification file (.tsx) nor a tagger probability file (.prob). Use --tsxfile or --probfile to provide one of them\n";
+    cerr<<"Error: You have provided neither a tagger specification file (.tsx) nor a tagger probability file (.prob). Use --tsxfile or --probfile to provide one of them\n";
     help(argv[0]);
     exit(EXIT_FAILURE);
   }
 
   if((tsxfile!="") && (probfile!="")) {
-    wcerr<<"Error: You provided a tagger specification file and a tagger probability file. Only one of them can be provided, not both\n";
+    cerr<<"Error: You provided a tagger specification file and a tagger probability file. Only one of them can be provided, not both\n";
     help(argv[0]);
     exit(EXIT_FAILURE);
   }
 
   if (tsxfile!="") {
-    wcerr<<"Reading tagger specification from file '"<<tsxfile<<"' ..."<<flush;
+    cerr<<"Reading tagger specification from file '"<<tsxfile<<"' ..."<<flush;
     TSXReader treader;
     treader.read(tsxfile);
     tagger_data_hmm=treader.getTaggerData();
-    wcerr<<"done.\n";
+    cerr<<"done.\n";
     check_ambclasses=false;
   }
 
   if (probfile!="") {
-    wcerr<<"Reading tagger parameters from file '"<<probfile<<"' ..."<<flush;
+    cerr<<"Reading tagger parameters from file '"<<probfile<<"' ..."<<flush;
     FILE* fin=NULL;
     fin=fopen(probfile.c_str(), "r");
     check_file(fin, probfile);
     tagger_data_hmm.read(fin);
-    wcerr<<"done.\n";
+    cerr<<"done.\n";
     fclose(fin);
     check_ambclasses=true;
   }
 
   TaggerWord::setArrayTags(tagger_data_hmm.getArrayTags());
 
-  readwords(stdin, corpus_length);
+  readwords(corpus_length);
 }
