@@ -9,7 +9,7 @@ StreamTagger::~StreamTagger() {}
 
 void StreamTagger::outputLexicalUnit(
     const LexicalUnit &lexical_unit, const Optional<Analysis> analysis,
-    std::wostream &output) {
+    std::ostream &output) {
   Stream::outputLexicalUnit(lexical_unit, analysis, output, TheFlags);
 }
 }
