@@ -1,5 +1,5 @@
 if [[ $# != 1 ]]; then
-    echo "USAGE: $(basename $0) <input_file>"
+    echo "USAGE: $(basename "$0") <input_file>"
     exit 1
 fi
 
