@@ -8,6 +8,10 @@ from os import devnull
 from os.path import join as pjoin
 from os.path import abspath, dirname
 from subprocess import check_call, Popen, PIPE, CalledProcessError
+from sys import stderr
+
+
+import signal
 
 
 # Utilities
@@ -378,3 +382,77 @@ class FilterAmbiguityTest(unittest.TestCase):
         self.assertEqual('^아/아<noun>$\n', output)
         stderr = open(stderrpath, 'r').read().strip()
         self.assertEqual('', stderr)
+
+class Alarm(Exception):
+    pass
+
+class PerceptronNullFlushTest(unittest.TestCase):
+
+    inputs = [
+        "^this/this<prn><dem><mf><sg>$ ^here/here<adv>$",
+        "^is/be<vbser><pres><p3><sg>$ ^a/a<det><ind><sg>$",
+        "^little/little<adj>/little<adv>$",
+        "^flushing/*flushing$ ^test/test<n><sg>/test<vblex><inf>/test<vblex><pres>/test<vblex><imp>$"
+    ]
+
+    outputs = [
+        "^this<prn><dem><mf><sg>$ ^here<adv>$",
+        "^be<vbser><pres><p3><sg>$ ^a<det><ind><sg>$",
+        "^little<adv>$",
+        "^*flushing$ ^test<n><sg>$"
+    ]
+
+
+    def alarmHandler(self, signum, frame):
+        raise Alarm
+
+    def withTimeout(self, seconds, cmd, *args, **kwds):
+        signal.signal(signal.SIGALRM, self.alarmHandler)
+        signal.alarm(seconds)
+        ret = cmd(*args, **kwds)
+        signal.alarm(0)         # reset the alarm
+        return ret
+
+    def communicateFlush(self, string, process):
+        if string:
+            process.stdin.write(string.encode('utf-8'))
+            process.stdin.write(b'\0')
+            process.stdin.flush()
+
+        output = []
+        char = None
+        try:
+            char = self.withTimeout(2, process.stdout.read, 1)
+        except Alarm:
+            print("Timeout before reading a single character!", file=stderr)
+        while char and char != b'\0':
+            output.append(char)
+            try:
+                char = self.withTimeout(2, process.stdout.read, 1)
+            except Alarm:
+                print("Timeout before reading %s chars" % len(output),
+                        file=stderr)
+                break           # send what we got up till now
+
+        return b"".join(output).decode('utf-8').replace('\r\n', '\n')
+
+    def test_null_flush(self):
+        try:
+            proc = Popen([APERTIUM_TAGGER, '-xg', '-z', "data/eng.prob"],
+                                stdin=PIPE,
+                                stdout=PIPE,
+                                stderr=PIPE)
+
+            for inp, exp in zip(self.inputs, self.outputs):
+                output = self.communicateFlush(inp+"[][\n]", proc)
+                self.assertEqual(output, exp+"[][\n]")
+
+            proc.communicate() # let it terminate
+            proc.stdin.close()
+            proc.stdout.close()
+            proc.stderr.close()
+            retCode = proc.poll()
+            self.assertEqual(retCode, 0)
+
+        finally:
+            pass
